//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "Radar.h"
#pragma package(smart_init)
//---------------------------------------------------------------------------
// ValidCtrCheck is used to assure that the components created do not have
// any pure virtual functions.
//

static inline void ValidCtrCheck(TRadar *)
{
    new TRadar(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TTarget::Assign(TPersistent* Value)
{
  Bearing = dynamic_cast<TTarget*>(Value)->Bearing;
  Distance = dynamic_cast<TTarget*>(Value)->Distance;
  Color=dynamic_cast<TTarget*>(Value)->Color;
  DrawTarget=dynamic_cast<TTarget*>(Value)->DrawTarget;
}
void __fastcall TSound::Assign(TPersistent *Value)
{
UserSoundFile= dynamic_cast<TSound*>(Value)->UserSoundFile;
  Enabled = dynamic_cast<TSound*>(Value)->Enabled;
  Options=dynamic_cast<TSound*>(Value)->Options;
}



__fastcall TRadar::TRadar(TComponent* Owner)
    : TCustomControl(Owner)
{

FInterval=1000;
FAngle=180;
FRange=1000;
FTimer=new TTimer(this);
FTimer->Enabled=true;
FTimer->OnTimer=RestartTimer;
FPen = new TPen();
FPen->OnChange = StyleChanged;
FBrush = new TBrush();
FTarget= new TTarget();
FSound= new TSound();
FSound->Enabled=true;
FTarget->DrawTarget=true;
FSound->Options=soSystemOk;
FBrush->OnChange = StyleChanged;
Width=100;
Height=100;
DragMode=dmAutomatic;
Invalidate();
}

//---------------------------------------------------------------------------
__fastcall TRadar::~TRadar(void)
{
  delete FPen;
  delete FBrush;
  delete FTimer;
  delete FTarget;
  delete FSound;
}
//---------------------------
void __fastcall TRadar::StyleChanged(TObject* /*Sender*/)
{
   Invalidate();
}

void __fastcall TRadar::SetBrush(TBrush* Value)
{
  FBrush->Assign(Value);
}

void __fastcall TRadar::SetPen(TPen* Value)
{
  FPen->Assign(Value);
}
void __fastcall TRadar::SetTarget(TTarget* Value)
{
FTarget->Assign(Value);
}
void __fastcall TRadar::SetSound(TSound* Value)
{
 FSound->Assign(Value);
}

void __fastcall TRadar::SetAngle(int Value)
{
//limit angles to between 0 and 360 degs
 if(Value !=FAngle)
 {
 if(Value>360)Value=0;
 if(Value<0)Value=360;
 FAngle=Value;
 Invalidate();
 } }
void __fastcall TRadar::SetEnabled(bool Value)
{
 if(Value !=FEnabled){
  FEnabled=Value;
  FTimer->Enabled=FEnabled ;
  }
}
void __fastcall TSound::SetOptions(TSoundOptions ASound)
{
if(ASound !=FOptions)FOptions=ASound;
}
void __fastcall TSound::SetEnabled(bool Value)
{
 if(Value!=FEnabled)FEnabled=Value;
}
void __fastcall TSound::SetSoundFile(AnsiString Value)
{

if(Value !=FSoundFile)FSoundFile=Value;
}

void __fastcall TRadar::SetInterval(int Value)
{
if(Value !=FInterval)FInterval=Value;
FTimer->Interval=FInterval;
}
void __fastcall TRadar::RestartTimer(TObject*/*Sender*/)
{
int ang=FAngle;
//increment the angle by 5degs to keep the speed up
SetAngle(ang+5);
FTimer->Enabled=FEnabled;

}
void __fastcall TRadar::SetBackColor(TColor Value)
{
if(Value !=FBackColor)FBackColor=Value;
Invalidate();
}


void __fastcall TTarget::SetBearing(int Value)
{
if(Value !=FBearing)
{
//only allow bearings between 0 and 360 degs
if(Value>360)Value=360;
if(Value<0)Value=0;
FBearing=Value;
}
}
void __fastcall TTarget::SetDistance(int Value)
{
if(Value !=FDistance)
FDistance=Value;
}

//-------------------------------------------------------




void __fastcall TRadar::Paint(void)
{
//Set up an offscreen Bitmap
BackDraw=new Graphics::TBitmap();
BackDraw->Width=Width;
BackDraw->Height=Height;
BackDraw->Canvas->Brush->Color = FBackColor;
//======
  //draw the ellipses and cross hairs
  BackDraw->Canvas->Brush->Style = bsSolid;
  BackDraw->Canvas->FillRect(Rect(0, 0, Width, Height));
  BackDraw->Canvas->Pen = FPen;
  BackDraw->Canvas->Brush = FBrush;
  BackDraw->Canvas->Ellipse(0,0,0+Width,0+Height);
  BackDraw->Canvas->Ellipse(0+Width/10,0+Height/10,Width-Width/10,Height-Height/10);
  BackDraw->Canvas->Ellipse(0+Width/5,0+Height/5,Width-Width/5,Height-Height/5);
  BackDraw->Canvas->Ellipse(0+Width/3,0+Height/3,Width-Width/3,Height-Height/3);
  BackDraw->Canvas->MoveTo(0,Height/2);
  BackDraw->Canvas->LineTo(Width,Height/2);
  BackDraw->Canvas->MoveTo(Width/2,0);
  BackDraw->Canvas->LineTo(Width/2,Height);
  BackDraw->Canvas->MoveTo(Width/2,Height/2);
 //=======
// calculate the angle and bearing of the sweep hand and draw it
float  sX = sin((FAngle / 180.0) * PI);
float  sY = cos((FAngle / 180.0) * PI);
int stX = floor(sX * Width/2);
int stY = floor(sY * Height/2);
BackDraw->Canvas->LineTo(Width/2 + stX, Height/2 - stY);
//=============


// make sure the target is within 5degs of the sweep hand and that it is within range
if(FAngle<FTarget->Bearing+3 && FAngle>FTarget->Bearing-3 && FTarget->Distance<FRange)
//=============
{
// set up the event handler
if(OnAquireTarget)OnAquireTarget(this);
//Make sure sound is enabled,determine which sound is to
//be played and play it.
if(FSound->Enabled)
{
int x;
 switch(FSound->Options)
    {
    case 0:{x=FSound->Options*16;MessageBeep(x);}break;
    case 1:{x=FSound->Options*16;MessageBeep(x);}break;
    case 2:{x=FSound->Options*16;MessageBeep(x);}break;
    case 3:{x=FSound->Options*16;MessageBeep(x);}break;
    case 4:{x=FSound->Options*16;MessageBeep(x);}break;
    case 5:{x=0xFFFFFFFF;MessageBeep(x);}break;
    case 6:{char *Snd=FSound->UserSoundFile.c_str();PlaySound(Snd,NULL,SND_ASYNC);}break;
}
    }

//==========
if(FTarget->DrawTarget)  // make sure Draw Target is Enabled
{
// calculate the position (angle and distance) of the target and draw it.

float dWidth=(float (Width)/float (FRange))*FTarget->Distance;
float dHeight=(float(Height)/float(FRange))*FTarget->Distance;
int dX=floor(sX*dWidth/2);
int dY=floor(sY*dHeight/2);
int TargetWidth=Width/20;
int TargetHeight=Height/20;
BackDraw->Canvas->Brush->Color=FTarget->Color;
BackDraw->Canvas->Ellipse((Width/2+dX)+TargetWidth,(Height/2-dY)+TargetHeight,
(Width/2+dX)-TargetWidth,(Height/2-dY)-TargetHeight);
BackDraw->Canvas->Brush=FBrush;
}
}

//Draw the Bitmap (BackDraw) on the component's canvas
Canvas->Draw(0,0,BackDraw);
delete BackDraw;
}
namespace Radar
{
    void __fastcall PACKAGE Register()
    {
        TComponentClass classes[1] = {__classid(TRadar)};
        RegisterComponents("Samples", classes, 0);
    }


}
//---------------------------------------------------------------------------
